<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
/**
 *
 * course_format_iena_sections
 *
 * @package    format_iena
 * @copyright  2018 Softia/Université lorraine
 * @author     vrignaud camille / Michaël Lebeau
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

namespace format_iena\output;



class course_format_iena_table_option {

    const DISPLAY_GROUPS= "display_groups";
    const DISPLAY_CUSTOM= "display_custom";
    const DISPLAY_DETAILS= "display_details";
    const DISPLAY_STATUS="display_status";


    /** @var boolean */
    
    /**
     * **/

    public $optionname;
    public $optionvalue;
    public $courseid;
    public $userid;
    public $listoptions=array(
        self::DISPLAY_GROUPS=>array("value"=>0),
        self::DISPLAY_CUSTOM=>array("value"=>0),
        self::DISPLAY_DETAILS=>array("value"=>0),
        self::DISPLAY_STATUS=>array("value"=>0)
    );

    
    /**
     * [__construct description]
     * @param boolean $displaygroups  [description]
     * @param boolean $displaycustom  [description]
     * @param boolean $displaydetails [description]
     * @param boolean $displaystatus  [description]
     * @param int     $idcourse       [description]
     * @param int     $iduser         [description]
     */
    public function __construct(string $option = null, int $value=0, int $courseid = 0, int $userid= 0){
        $this->optionname=$option;
        $this->optionvalue=$value;
        $this->courseid=$courseid;
        $this->userid=$userid;
    }

    /**
     * 
     * @return array
     * @throws dml_exception
     */
    public function get_options_by_course_id($courseid) {
        global $DB;
        $requete=[];
        $requete = $DB->get_records('format_iena_options', ['courseid' => $courseid]);
        return $requete;
    }



    public function create_array_option(){
        global $DB;
        $this->timecreated=time();

         $arrayoptions=$DB->insert_record("format_iena_options", $this, true);
         return $arrayoptions;
    }

    public function get_request_options(){
         global $DB;
 
         $arrayoptions = $DB->get_records('format_iena_options', ['courseid' => $this->courseid]);
         $this->set_array_options(array_values($arrayoptions));
         //return $this->get_array_options(array_values($arrayoptions));
    }

    /**
     * formate le résultat de la requête qui récupère les options existantes
     * @param  [array] $requestoptions [le tableau de résultat de la requête]
     * @return [array]                 [un tableau d'objet qui contient nomoption => ["value" => value, "id"=>idoption]]
     */
    private function set_array_options($requestoptions){
        if(is_object($requestoptions)){
             $this->listoptions[$requestoptions->optionname]['value']=$requestoptions['optionvalue'];
             $this->listoptions[$requestoptions->optionname]['id']=$requestoptions['id'];
        }
        else{

            for($i=0; $i<count($requestoptions); $i++){
                $this->listoptions[$requestoptions[$i]->optionname]['value']=$requestoptions[$i]->optionvalue;
                $this->listoptions[$requestoptions[$i]->optionname]['id']=$requestoptions[$i]->id;
            }
        }
        
        //return $this->listoptions;
    }

    public function toArray() {
        $vars = [];
        foreach ($this as $varname => $varvalue) {

            $vars[$varname] = $varvalue;
        }
        return $vars;
    }

    public function get_list_options(){
        return $this->listoptions;
    }
 

    public function save_property($property, $value, $existingoption =false){
        global $DB;
        $this->setOptionname($property);
        $this->setOptionvalue($value);
        if($existingoption){
            $this->timemodified=time();
            $this->id=$existingoption;
            return $result=$DB->update_record("format_iena_options", $this, true);
        }
        else{
            $this->timecreated=time();
            $this->timemodified=null;
            return $result=$DB->insert_record("format_iena_options", $this, true);
        }
        
        

    }

    public function check_option_checked($optionfromDB, $arrayfrominput){

    }

    public function check_option_unchecked(){

    }

    public function check_options_state($arrayfrominput){
        foreach($this->listoptions as $option=>$arrayvalues){
            if(isset($arrayvalues["id"])){
                $this->save_property($option, $arrayfrominput->$option, $arrayvalues["id"]);
            }
            else if($arrayfrominput->$option != 0){
                $this->save_property($option, $arrayfrominput->$option);
            }
        }
    }
  


    /**
     * @return mixed
     */
    public function getOptionname()
    {
        return $this->optionname;
    }

    /**
     * @param mixed $optionname
     *
     * @return self
     */
    public function setOptionname($optionname)
    {
        $this->optionname = $optionname;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getOptionvalue()
    {
        return $this->optionvalue;
    }

    /**
     * @param mixed $optionvalue
     *
     * @return self
     */
    public function setOptionvalue($optionvalue)
    {
        $this->optionvalue = $optionvalue;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getCourseid()
    {
        return $this->courseid;
    }

    /**
     * @param mixed $courseid
     *
     * @return self
     */
    public function setCourseid($courseid)
    {
        $this->courseid = $courseid;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getUserid()
    {
        return $this->userid;
    }

    /**
     * @param mixed $userid
     *
     * @return self
     */
    public function setUserid($userid)
    {
        $this->userid = $userid;

        return $this;
    }

    /**
     * @return mixed
     */
    public function getListoptions()
    {
        return $this->listoptions;
    }

    /**
     * @param mixed $listoptions
     *
     * @return self
     */
    public function setListoptions($listoptions)
    {
        $this->listoptions = $listoptions;

        return $this;
    }
}
