<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.
/**
 *
 * @package    format_iena
 * @copyright  2018 Softia/Université lorraine
 * @author     vrignaud camille
 * @license    http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

define('NO_OUTPUT_BUFFERING', true);
require_once('../../../config.php');

global $COURSE, $DB, $USER, $CFG;

$courseid = required_param('courseid', PARAM_INT);
$url = new moodle_url('/course/format/iena/send_message.php', array('courseid' => $courseid));

$PAGE->set_pagelayout('course');
$PAGE->set_url($url);

if (!has_capability('course/iena:suivi',
    $context = context_course::instance($courseid), $USER->id)) {
    $link = $CFG->wwwroot . '/course/view.php?id=' . $courseid;
    header("Location: {$link}");
    exit;
}

$course = $DB->get_record('course', array('id' => $courseid), '*', MUST_EXIST);
require_login($course, false, null);

$students = "";
$backurl = $CFG->wwwroot . '/course/view.php?id=' . $courseid;

if (isset($_POST["students"])) {
    $students = $_POST["students"];
}
if (isset($_POST["back_url"])) {
    $backurl = $_POST["back_url"];
}

$data = new stdClass();
$nb = $students != "" ? count( explode(",", $students) ) : 0;

require_once('entity/course_format_iena_message.php');
$sendmessage = new course_format_iena_message(null, array('back_url' => $backurl,
    'students' => $students));

if ($nb == 0) {
    $data->nb_results = 0;
} else {
    $data->nb_results=$nb;
    $data->users_id = $students;
    $courseid = required_param('courseid', PARAM_INT);
    $data->link_cancel = $CFG->wwwroot . "/course/view.php?id=" . $courseid;
    $data->form = $sendmessage->render();
}

// If form is cancelled, we go back to the table.
// Else, we send the message and then go back to the table, with a success or failure message.
$backurl = $CFG->wwwroot . '/course/format/iena/suivi_unit.php'.$backurl;
if ($sendmessage->is_cancelled()) {
    header("Location: {$backurl}");
} else if ($fromform = $sendmessage->get_data()) {
    
    try {
        $messagecontent = $fromform->summary['text'];

        $usersid = explode(",", $fromform->students);

        $messagecontent .= "<br>". get_string('course', 'format_iena') . " :  <a href='" . $CFG->wwwroot .
            "/course/view.php?id=" . $course->id . "'>" . $course->fullname . "</a>";
        $messagecontent .= "<br> ". get_string('prof', 'format_iena') . " : " .
            $USER->firstname . " " . $USER->lastname;
        $brutmessagecontent = $course->fullname . " " . $_POST['summary']['text'];
        $coursectx = context_course::instance($courseid);
        $students = get_enrolled_users($coursectx);

        // Sending a notification to each students.
        foreach ($usersid as $userid) {
            foreach ($students as $student) {
                if ($student->id == $userid) {
                    $message = new \core\message\message();
                    $message->courseid = $course->id;
                    $message->component = 'moodle';
                    $message->name = 'instantmessage';
                    $message->userfrom = $USER;
                    $message->notification = '0';
                    $message->fullmessageformat = FORMAT_HTML;
                    $message->subject = "Rappel : " . $course->fullname;
                    $message->smallmessage = $messagecontent;
                    $message->fullmessage = $brutmessagecontent;
                    $message->fullmessagehtml = $messagecontent;
                    $message->userto = $student;
                    $messageid = message_send($message);
                    break;
                }
            }
        }

        $backurl .= "&msg_success=true";
    } catch (Exception $e) {
        $backurl .= "&msg_failure=true";
    }

    header("Location: {$backurl}");
} else {
    echo $OUTPUT->header();
    $renderer = $PAGE->get_renderer('format_iena');
    $renderer->display_message($data);
    echo $OUTPUT->footer();
}
